"""Doc of bioat.

"""
from __future__ import annotations, absolute_import
from bioat.about import about
from bioat.bam import Bam
from bioat.fastx import Fastx
from bioat.hic import HiC
from bioat.logger import set_logging_level
from bioat.mgi import Mgi
from bioat.system import System
from bioat.table import Table
from bioat.version import (__version__, __author__, __doc_format__)

__name__ = "bioat"
