from __future__ import absolute_import
from bioat.version import (__version__, __author__, __upgrade_date__, __name__, __email__, __doc_format__)

about = f"""
BioinformaticAnalysisTools ({__name__})
    - bioat version:
        {__version__}
    - repository address:
        https://github.com/hermanzhaozzzz/BioinformaticAnalysisTools

    - author:
        - name: {__author__}
        - email: {__email__}
    - doc:
        - format: {__doc_format__}
    - last update:
        - {__upgrade_date__}
"""
